/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.client.common;

import java.io.File;
import java.io.FileFilter;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appspy.client.common.config.CollectorConfig;
import org.appspy.client.common.config.ConfigLoader;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class CollectorFactory
{
	public static final String PROPERTIES_CONFIG_LOADER_PARAM_NAME = "org.appspy.propertiesConfigLoader";
	public static final String COLLECTOR_CONFIG_LOADER_PARAM_NAME = "org.appspy.collectorConfigLoader";
	
	public static final String DEFAULT_PROPERTIES_CONFIG_LOADER = org.appspy.client.common.config.FileConfigLoader.class.getName();
	public static final String DEFAULT_COLLECTOR_CONFIG_LOADER = org.appspy.client.common.config.FileConfigLoader.class.getName();
	
	public static final String PROPERTIES_CONFIG_PATTERN_PARAM_NAME = "org.appspy.propertiesConfigPattern";
	public static final String COLLECTOR_CONFIG_PATTERN_PARAM_NAME = "org.appspy.collectorConfigPattern";
	
	public static final String DEFAULT_PROPERTIES_CONFIG_PATTERN = "appspy-config-{0}.properties";
	public static final String DEFAULT_COLLECTOR_CONFIG_PATTERN = "appspy-collector-{0}.xml";
	
	protected static Log sLog = LogFactory.getLog(CollectorFactory.class);
	
	protected static Integer sLock = new Integer(0);
	
	protected static Collector sCollector = null;
	protected static CollectorInfo sCollectorInfo = null;
	protected static CollectorConfig sCollectorConfig = null;

	public static Collector initCollector(Properties configProperties, URL collectorConfigURL) throws Exception
	{
		sCollectorInfo = new CollectorInfo();
		
		if (configProperties == null)
		{
			configProperties = getDefaultConfigProperties();
		}
		if (collectorConfigURL == null)
		{
			collectorConfigURL = getDefaultCollectorConfigURL();
		}
		
		sCollectorConfig = new CollectorConfig(configProperties);

		sLog.info("Creation of the collector");
		if (sLog.isInfoEnabled())
		{
			sLog.info("Using appspy dir : " + getAppspyDir());
		}

		File appSpyLibDir = new File(getAppspyDir(), "client/core/lib");

		File[] libs = appSpyLibDir.listFiles(new FileFilter() {

			public boolean accept(File pathname)
			{
				if (pathname.isFile() && pathname.getName().endsWith(".jar"))
				{
					return true;
				}
				return false;
			}
		});

		ArrayList<URL> urls = new ArrayList<URL>();

		for (File file : libs)
		{
			try
			{
				urls.add(file.toURL());
			} catch (MalformedURLException e)
			{
				e.printStackTrace();
			}
		}

		if (sLog.isInfoEnabled())
		{
			sLog.info("Libraries used in the collector classloader : " + urls);
		}
		URLClassLoader cl = new URLClassLoader(urls.toArray(new URL[] {}), Thread.currentThread().getContextClassLoader());
		try
		{
			Class<?> appSpyUtilClass = Class.forName("org.appspy.client.collector.CollectorFactoryUtils", true, cl);
			Method m = appSpyUtilClass.getMethod("getCollector", new Class[] {Properties.class, URL.class});
			sCollector = (Collector) m.invoke(null, new Object[] {configProperties, collectorConfigURL});
		} catch (Exception e)
		{
			sLog.error("Error in Collector initialization", e);
			throw e;
		}

		return sCollector;
	}
	
	public static Collector getCollector()
	{
		if (sCollectorInfo == null)
		{
			synchronized (sLock)
			{
				if (sCollectorInfo == null)
				{
					try
					{
						sCollector = initCollector(null, null);
					}
					catch (Exception ex)
					{
						sCollectorInfo.setStatus(CollectorInfo.STATUS_BAD_CONFIG, ex);
					}
				}
			}
		}
		
		return sCollector;
	}

	public static String getAppspyDir()
	{
		return System.getProperty("org.appspy.dir", System.getProperty("user.home") + "/appspy");
	}

	public static String getAppspyMode()
	{
		return System.getProperty("org.appspy.mode", "default");
	}
	
	protected static Properties getDefaultConfigProperties() throws Exception
	{
		String configLoaderClassName = System.getProperty(PROPERTIES_CONFIG_LOADER_PARAM_NAME, DEFAULT_PROPERTIES_CONFIG_LOADER); 
		ConfigLoader configLoader = (ConfigLoader) Class.forName(configLoaderClassName, true, Thread.currentThread().getContextClassLoader()).newInstance();
		
		String configFileNamePattern = System.getProperty(PROPERTIES_CONFIG_PATTERN_PARAM_NAME, DEFAULT_PROPERTIES_CONFIG_PATTERN);
		String configFileName = MessageFormat.format(configFileNamePattern, CollectorFactory.getAppspyMode());
		
		return configLoader.loadConfig(configFileName);
	}

	protected static URL getDefaultCollectorConfigURL() throws Exception
	{
		String configLoaderClassName = System.getProperty(COLLECTOR_CONFIG_LOADER_PARAM_NAME, DEFAULT_COLLECTOR_CONFIG_LOADER); 
		ConfigLoader configLoader = (ConfigLoader) Class.forName(configLoaderClassName, true, Thread.currentThread().getContextClassLoader()).newInstance();
		
		String configFileNamePattern = System.getProperty(COLLECTOR_CONFIG_PATTERN_PARAM_NAME, DEFAULT_COLLECTOR_CONFIG_PATTERN);
		String configFileName = MessageFormat.format(configFileNamePattern, CollectorFactory.getAppspyMode());
		
		return configLoader.getURL(configFileName);
	}

	public static CollectorInfo getCollectorInfo()
	{
		if (sCollectorInfo == null)
		{
			getCollector();
		}
		return sCollectorInfo;
	}
	
	public static CollectorConfig getCollectorConfig()
	{
		if (sCollectorInfo == null)
		{
			getCollector();
		}
		return sCollectorConfig;
	}
}
